# Heroku CLI Data Beta Plugin

Heroku CLI Plugin for beta Heroku Data features.

[![Version](https://img.shields.io/npm/v/@heroku-cli/plugin-data-beta.svg)](https://www.npmjs.com/package/@heroku-cli/plugin-data-beta)
[![Downloads/week](https://img.shields.io/npm/dw/@heroku-cli/plugin-data-beta.svg)](https://npmjs.org/package/@heroku-cli/plugin-data-beta)
[![License](https://img.shields.io/npm/l/@heroku-cli/plugin-data-beta.svg)](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/master/package.json)

<!-- toc -->
* [Heroku CLI Data Beta Plugin](#heroku-cli-data-beta-plugin)
* [Installation](#installation)
* [Usage](#usage)
* [Commands](#commands)
<!-- tocstop -->

# Installation
```sh-session
$ heroku plugins:install data-beta
```

# Usage
<!-- usage -->
```sh-session
$ npm install -g @heroku-cli/plugin-data-beta
$ heroku COMMAND
running command...
$ heroku (--version)
@heroku-cli/plugin-data-beta/0.0.1 darwin-x64 node-v20.18.3
$ heroku --help [COMMAND]
USAGE
  $ heroku COMMAND
...
```
<!-- usagestop -->

# Commands
<!-- commands -->
* [`heroku data:pg:create TIER`](#heroku-datapgcreate-tier)
* [`heroku data:pg:destroy ADDON`](#heroku-datapgdestroy-addon)
* [`heroku data:pg:docs`](#heroku-datapgdocs)
* [`heroku data:pg:info ADDON`](#heroku-datapginfo-addon)
* [`heroku data:pg:psql DATABASE`](#heroku-datapgpsql-database)
* [`heroku data:pg:scale ADDON`](#heroku-datapgscale-addon)

## `heroku data:pg:create TIER`

Create a postgres performance addon

```
USAGE
  $ heroku data:pg:create TIER -a <value> [--level <value>] [--name <value>] [--as <value>] [--wait]
    [--high-availability] [--network private|shield] [--readers <value>]

ARGUMENTS
  TIER  (performance) Postgres tier

FLAGS
  -a, --app=<value>             (required) app to run command against
      --as=<value>              Name for the initial add-on attachment
      --[no-]high-availability  Enable a high availability cluster by provisioning a warm standby writer.
      --level=<value>           Set compute scale
      --name=<value>            Name for the add-on resource
      --network=<option>        Networking for the add-on
                                <options: private|shield>
      --readers=<value>         Provisions a reader instance pool with the specified number of instances
      --wait                    Watch add-on creation status and exit when complete

DESCRIPTION
  Create a postgres performance addon

EXAMPLES
  $ heroku data:pg:create performance --level 4G-Performance
```

_See code: [src/commands/data/pg/create.ts](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/v0.0.1/src/commands/data/pg/create.ts)_

## `heroku data:pg:destroy ADDON`

Destroy a postgres performance database

```
USAGE
  $ heroku data:pg:destroy ADDON [-f] [-c <value>] [--wait] [-a <value>]

ARGUMENTS
  ADDON  Name of the database, its attachment name, or related config var on an app

FLAGS
  -a, --app=<value>      app to run command against
  -c, --confirm=<value>  Skip confirmation prompts by passing in app name
  -f, --force            allow destruction even if connected to other apps
      --wait             watch database destruction status and exit when complete

DESCRIPTION
  Destroy a postgres performance database

EXAMPLES
  $ heroku data:pg:destroy database_name
```

_See code: [src/commands/data/pg/destroy.ts](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/v0.0.1/src/commands/data/pg/destroy.ts)_

## `heroku data:pg:docs`

Opens documentation for Heroku Postgres in your web browser.

```
USAGE
  $ heroku data:pg:docs [--browser <value>]

FLAGS
  --browser=<value>  browser to open docs with (example: "firefox", "safari")

DESCRIPTION
  Opens documentation for Heroku Postgres in your web browser.
```

_See code: [src/commands/data/pg/docs.ts](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/v0.0.1/src/commands/data/pg/docs.ts)_

## `heroku data:pg:info ADDON`

Get Postgres Addon Info

```
USAGE
  $ heroku data:pg:info ADDON [-a <value>]

ARGUMENTS
  ADDON  Name of the addon, its attachment name, or related config var on an app

FLAGS
  -a, --app=<value>  app to run command against

DESCRIPTION
  Get Postgres Addon Info

EXAMPLES
  $ heroku data:pg:info addon_name
```

_See code: [src/commands/data/pg/info.ts](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/v0.0.1/src/commands/data/pg/info.ts)_

## `heroku data:pg:psql DATABASE`

open a psql shell to the database

```
USAGE
  $ heroku data:pg:psql DATABASE -a <value> [-c <value>] [--credential <value>] [-f <value>] [-p <value>] [-r
    <value>]

ARGUMENTS
  DATABASE  Name of the addon, its attachment name, or related config var on an app

FLAGS
  -a, --app=<value>         (required) app to run command against
  -c, --command=<value>     SQL command to run
  -f, --file=<value>        SQL file to run
  -p, --pool=<value>        reader instance pool to connect to (only Performance-tier databases)
  -r, --remote=<value>      git remote of app to use
      --credential=<value>  credential to use

DESCRIPTION
  open a psql shell to the database
```

_See code: [src/commands/data/pg/psql.ts](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/v0.0.1/src/commands/data/pg/psql.ts)_

## `heroku data:pg:scale ADDON`

Scale your postgres addon

```
USAGE
  $ heroku data:pg:scale ADDON -a <value> [--count <value>] [--level <value>] [--pool <value>]
    [--high-availability] [--confirm <value>]

ARGUMENTS
  ADDON  Name of the addon to scale. If targeting an app, an env var associate with the addon may be used

FLAGS
  -a, --app=<value>             (required) app to run command against
      --confirm=<value>         Confirm scale action
      --count=<value>           Adjust the instance count of a pool
      --[no-]high-availability  Enable a high availability cluster by provisioning a warm standby writer
      --level=<value>           Adjust compute scale
      --pool=<value>            Adds a named instance pool when one does not exist or scales the instance pool which
                                exists based on level and instance flags

DESCRIPTION
  Scale your postgres addon

EXAMPLES
  $ heroku data:pg:scale
```

_See code: [src/commands/data/pg/scale.ts](https://github.com/heroku/heroku-cli-plugin-data-beta/blob/v0.0.1/src/commands/data/pg/scale.ts)_
<!-- commandsstop -->
