"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const color_1 = __importDefault(require("@heroku-cli/color"));
const core_1 = require("@oclif/core");
const command_1 = require("@heroku-cli/command");
const notify_1 = __importDefault(require("../../../lib/notify"));
const confirmCommand_1 = __importDefault(require("../../../lib/confirmCommand"));
const destroy_addon_1 = __importDefault(require("../../../lib/addons/destroy_addon"));
const resolveAddon_1 = require("../../../lib/data/pg/resolveAddon");
const baseCommand_1 = __importDefault(require("../../../lib/baseCommand"));
const util_1 = require("../../../lib/addons/util");
class Destroy extends baseCommand_1.default {
    static description = 'Destroy a postgres performance database';
    static examples = ['<%= config.bin %> <%= command.id %> database_name'];
    static args = {
        addon: core_1.Args.string({
            description: 'Name of the database, its attachment name, or related config var on an app',
            required: true,
        }),
    };
    static flags = {
        force: command_1.flags.boolean({ char: 'f', description: 'allow destruction even if connected to other apps' }),
        confirm: command_1.flags.string({ char: 'c', description: 'Skip confirmation prompts by passing in app name' }),
        wait: command_1.flags.boolean({ description: 'watch database destruction status and exit when complete' }),
        app: command_1.flags.app(),
    };
    async run() {
        const { args, flags } = await this.parse(Destroy);
        const { app, wait, confirm } = flags;
        const { addon: addonName } = args;
        const force = flags.force || process.env.HEROKU_FORCE === '1';
        const addon = await (0, resolveAddon_1.resolveAddon)(this.heroku, addonName, flags.app);
        // prevent deletion of add-on when context.app is set but the addon is
        // attached to a different app
        const addonApp = addon.app?.name;
        const isAppMismatch = app && addonApp !== app;
        if (isAppMismatch) {
            core_1.ux.error(`${color_1.default.yellow(addon.name ?? '')} is on ${color_1.default.magenta(addonApp ?? '')} not ${color_1.default.magenta(app)}`);
        }
        if (!(0, util_1.performancePgPlan)(addon)) {
            core_1.ux.error('Cannot delete non performance databases');
        }
        await (0, confirmCommand_1.default)(addon.app?.name ?? '', confirm);
        try {
            await (0, destroy_addon_1.default)(this.heroku, addon, force, wait);
            if (wait) {
                (0, notify_1.default)(`heroku data:pg:destroy ${addon.name}`, 'Database successfully destroyed');
            }
        }
        catch (error) {
            if (wait) {
                (0, notify_1.default)(`heroku data:pg:destroy ${addon.name}`, 'Failed to destroy database', false);
            }
            throw error;
        }
    }
}
exports.default = Destroy;
