"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const color_1 = __importDefault(require("@heroku-cli/color"));
const command_1 = require("@heroku-cli/command");
const core_1 = require("@oclif/core");
const util_1 = require("../../../lib/addons/util");
const fetcher_1 = require("../../../lib/data/pg/psql/fetcher");
const psql_1 = require("../../../lib/data/pg/psql/psql");
class Psql extends command_1.Command {
    static args = {
        database: core_1.Args.string({
            description: 'Name of the addon, its attachment name, or related config var on an app',
            required: true,
        }),
    };
    static description = 'open a psql shell to the database';
    static flags = {
        app: command_1.flags.app({ required: true }),
        command: command_1.flags.string({ char: 'c', description: 'SQL command to run' }),
        credential: command_1.flags.string({ description: 'credential to use' }),
        file: command_1.flags.string({ char: 'f', description: 'SQL file to run' }),
        pool: command_1.flags.string({
            char: 'p',
            description: 'reader instance pool to connect to (only Performance-tier databases)',
        }),
        remote: command_1.flags.remote(),
    };
    async run() {
        const { flags, args } = await this.parse(Psql);
        const { database: databaseArg } = args;
        const { app, command, credential, file, pool } = flags;
        const namespace = credential ? `credential:${credential}` : undefined;
        let db;
        try {
            db = await (0, fetcher_1.database)(this.heroku, app, databaseArg, namespace, pool);
        }
        catch (error) {
            if (namespace && error instanceof Error && error.message === "Couldn't find that addon.") {
                throw new Error("Credential doesn't match, make sure credential is attached");
            }
            throw error;
        }
        if ((0, util_1.performancePrivatePlan)(db.attachment.addon)) {
            if (file)
                core_1.ux.error('Privately networked Performance databases currently don’t support SQL file execution', { exit: 1 });
            let psqlCommand;
            if (command) {
                psqlCommand = `psql -c "${command.replace(/"/g, '\\"')}" --set sslmode=require $${db.connStringVar}`;
            }
            else {
                const prompt = `${db.attachment.app.name}::${db.connStringVar.slice(0, -4)}%R%# `;
                psqlCommand = `psql --set PROMPT1="${prompt}" --set PROMPT2="${prompt}" --set sslmode=require $${db.connStringVar}`;
            }
            const opts = {
                'exit-code': true,
                'no-tty': false,
                app,
                attach: true,
                command: psqlCommand,
                env: `PGAPPNAME='psql ${command ? 'non-' : ''}interactive';PGSSLMODE=require`,
                heroku: this.heroku,
                notify: false,
                showStatus: false,
            };
            return (0, psql_1.runThroughOneOffDyno)(opts);
        }
        core_1.stderr.write(`--> Connecting to ${color_1.default.yellow(db.attachment.addon.name)}${pool ? color_1.default.dim(` (pool: ${pool})`) : ''}\n`);
        if (command) {
            const output = await (0, psql_1.exec)(db, command);
            process.stdout.write(output);
        }
        else if (file) {
            const output = await (0, psql_1.execFile)(db, file);
            process.stdout.write(output);
        }
        else
            await (0, psql_1.interactive)(db);
    }
}
exports.default = Psql;
