"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const command_1 = require("@heroku-cli/command");
const color_1 = __importDefault(require("@heroku-cli/color"));
const resolveAddon_1 = require("../../../lib/data/pg/resolveAddon");
const baseCommand_1 = __importDefault(require("../../../lib/baseCommand"));
const util_1 = require("../../../lib/addons/util");
const inquirer = __importStar(require("inquirer"));
const utils_1 = require("../../../lib/utils");
const tableHeaders = {
    factor: {},
    from: {},
    to: {},
};
const renderPoolScale = ({ count, ha, level }) => {
    const renderedHa = ha ? `(${'HA'})` : '';
    const rendered = `${count} @ ${level}${renderedHa}`;
    return rendered;
};
const renderPoolScaleChanges = ({ from, to, }) => {
    return `${from.count} @ ${from.level}${from.ha ? `(${'HA'})` : ''} -> ${to.count} @ ${to.level}${to.ha ? `(${'HA'})` : ''}`;
};
const getTableData = (scaleResponse) => {
    return scaleResponse.changes.map((change) => {
        switch (change.name) {
            case 'high availability':
                return {
                    factor: change.name,
                    from: color_1.default.yellow(change.previous ? 'enabled' : 'disabled'),
                    to: color_1.default.cyan(change.current ? 'enabled' : 'disabled'),
                };
            default:
                return {
                    factor: change.name,
                    from: color_1.default.yellow(change.previous),
                    to: color_1.default.cyan(change.current),
                };
        }
    });
};
class Scale extends baseCommand_1.default {
    static description = 'Scale your postgres addon';
    static examples = ['<%= config.bin %> <%= command.id %>'];
    static args = {
        addon: core_1.Args.string({
            description: 'Name of the addon to scale. If targeting an app, an env var associate with the addon may be used',
            required: true,
        }),
    };
    static flags = {
        app: command_1.flags.app({ required: true }),
        count: command_1.flags.integer({
            description: 'Adjust the instance count of a pool',
            min: 0,
            max: 15,
        }),
        level: command_1.flags.string({
            description: 'Adjust compute scale',
        }),
        pool: command_1.flags.string({
            description: 'Adds a named instance pool when one does not exist or scales the instance pool which exists based on level and instance flags',
        }),
        // flags are improperly typed. a boolean flag can be undefined if not passed
        'high-availability': command_1.flags.boolean({
            description: 'Enable a high availability cluster by provisioning a warm standby writer',
            allowNo: true,
        }),
        confirm: command_1.flags.string({
            description: 'Confirm scale action',
        }),
    };
    /**
     * Runs through interactive prompts to determine factors to scale
     */
    async interactive({ addon }) {
        const [infoRes, levelsRes] = await Promise.all([
            this.dataApi.get(`/data/postgres/v1/${addon.id}`),
            this.dataApi.get('/data/postgres/v1/levels/performance'),
        ]);
        const levels = levelsRes.body;
        const info = infoRes.body;
        const pools = info.pools;
        const levelNames = levels.map((level) => level.name);
        const scaleTargetMessage = 'Which instance pool would you like to scale?';
        const scaleTargetPoolLabels = (0, utils_1.alignColumns)(pools.map(({ name, count, level }) => [
            `${name}`,
            renderPoolScale({ count, level, ha: name === 'writer' && info.highly_available }),
        ]));
        const scaleTarget = (await inquirer.prompt({
            name: 'target',
            type: 'list',
            message: scaleTargetMessage,
            choices: [
                ...pools.map(({ name }, i) => ({
                    name: scaleTargetPoolLabels[i],
                    value: name,
                })),
                new inquirer.Separator('-'.repeat(scaleTargetMessage.length)),
                'Create a new instance pool',
            ],
            pageSize: 12,
            default: pools[0].name,
        })).target;
        let pool;
        if (scaleTarget === 'Create a new instance pool') {
            pool = (await inquirer.prompt({
                name: 'pool',
                type: 'input',
                message: 'What would you like to name your instance pool?',
                validate: (input) => {
                    if (!input) {
                        return 'Please enter an instance pool name';
                    }
                    else {
                        return true;
                    }
                },
            })).pool;
        }
        else {
            pool = scaleTarget;
        }
        const currentLevel = info.pools.find((p) => p.name === pool)?.level;
        const currentCount = info.pools.find((p) => p.name === pool)?.count;
        const isNewPool = info.pools.every((p) => p.name !== pool);
        const level = (await inquirer.prompt({
            name: 'level',
            type: 'list',
            message: 'Select a level',
            choices: levelNames,
            pageSize: 12,
            default: currentLevel ?? null,
        })).level;
        let count;
        if (pool !== 'writer') {
            count = Number((await inquirer.prompt({
                name: 'count',
                type: 'input',
                message: 'Select number of instances',
                validate: (value) => {
                    const parsedValue = parseInt(value);
                    if (isNaN(parsedValue)) {
                        return 'Please enter a valid integer';
                    }
                    return true;
                },
                default: isNewPool ? 1 : currentCount,
            })).count);
        }
        else {
            // we should have currentCount if the pool is 'writer'
            count = currentCount;
        }
        let enableHA = undefined;
        if (pool === 'writer' && !info.highly_available) {
            enableHA = (await inquirer.prompt({
                type: 'confirm',
                name: 'enableHA',
                message: 'Enable high availability with a hidden standby writer?',
                default: true,
            })).enableHA;
        }
        // confirmation prompt
        let confirmed = false;
        const confirmMessage = isNewPool
            ? `${'Continue creating new instance pool'} ${color_1.default.cyan(`${pool}: ${renderPoolScale({
                count,
                ha: false,
                level,
            })}`)}?`
            : `${'Continue scaling'} ${color_1.default.cyan(`${pool}: ${renderPoolScaleChanges({
                from: {
                    count: currentCount,
                    level: currentLevel,
                    ha: pool === 'writer' && info.highly_available,
                },
                to: {
                    count: count + (enableHA ? 1 : 0),
                    level: level,
                    ha: Boolean(enableHA) || (pool === 'writer' && info.highly_available),
                },
            })}`)}?`;
        confirmed = (await inquirer.prompt({
            type: 'confirm',
            name: 'confirm',
            message: confirmMessage,
        })).confirm;
        return {
            pool,
            // count represents the "selected" count... but for the writer pool we just grab the current count
            // this is a hack to prevent passing the count to the backend when targeting the writer pool, which is not current allowed
            count: pool !== 'writer' ? count : undefined,
            highAvailability: enableHA || undefined,
            level,
            confirmed,
        };
    }
    async run() {
        const { args, flags } = await this.parse(Scale);
        const { addon: addonName } = args;
        let { pool, level, count, 'high-availability': highAvailability } = flags;
        const addon = await (0, resolveAddon_1.resolveAddon)(this.heroku, addonName, flags.app);
        const interactive = level === undefined && count === undefined && highAvailability === undefined;
        if (interactive) {
            const promptsRes = await this.interactive({ addon });
            if (promptsRes.confirmed === false) {
                return;
            }
            ;
            ({ pool, level, count, highAvailability } = promptsRes);
        }
        const body = await (0, util_1.trapConfirmationRequired)(flags.app, flags.confirm, async (confirm) => {
            return (await this.dataApi.put(`/data/postgres/v1/${addon.id}/scale`, {
                body: {
                    followers: count,
                    level,
                    pool,
                    high_availability: highAvailability,
                    confirmed: confirm === flags.app,
                },
            })).body;
        });
        if (body.changes.length === 0) {
            core_1.ux.log(`\nNo updates applied to ${color_1.default.addon(addon.name)}.`);
        }
        else {
            const tableData = getTableData(body);
            // we can assume the all changes will have the same pool. Potential to improve the response shape in the future
            core_1.ux.log(`\nUpdates applied to the ${color_1.default.green(body.changes[0].pool)} instance pool on ${color_1.default.addon(addon.name)}:`);
            core_1.ux.table(tableData, tableHeaders);
            core_1.ux.log(`\nYour database changes will be applied shortly. You can use ${color_1.default.cmd(`data:pg:info ${addon.name} -a ${flags.app}`)} to track progress`);
        }
    }
}
exports.default = Scale;
