"use strict";
// https://github.com/heroku/cli/blob/main/packages/cli/src/lib/addons/addons_wait.ts
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.waitForAddonDeprovisioning = exports.waitForAddonProvisioning = void 0;
const core_1 = require("@oclif/core");
const color_1 = __importDefault(require("@heroku-cli/color"));
const waitForAddonProvisioning = async function (api, addon, interval) {
    const app = addon.app?.name || '';
    const addonName = addon.name;
    let addonBody = { ...addon };
    core_1.ux.action.start(`Creating ${color_1.default.addon(addonName || '')}`);
    while (addonBody.state === 'provisioning') {
        await new Promise(resolve => setTimeout(resolve, interval * 1000));
        const addonResponse = await api.get(`/apps/${app}/addons/${addonName}`, {
            headers: { 'Accept-Expansion': 'addon_service,plan' },
        });
        addonBody = addonResponse?.body;
    }
    if (addonBody.state === 'deprovisioned') {
        throw new Error(`The add-on was unable to be created, with status ${addonBody.state}`);
    }
    core_1.ux.action.stop();
    return addonBody;
};
exports.waitForAddonProvisioning = waitForAddonProvisioning;
const waitForAddonDeprovisioning = async function (api, addon, interval) {
    const app = addon.app?.name || '';
    const addonName = addon.name || '';
    let addonResponse = { ...addon };
    core_1.ux.action.start(`Destroying ${color_1.default.addon(addonName)}`);
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    while (addonResponse.state === 'deprovisioning') {
        await new Promise(resolve => setTimeout(resolve, interval * 1000));
        await api.get(`/apps/${app}/addons/${addonName}`, {
            headers: { 'Accept-Expansion': 'addon_service,plan' },
        }).then(response => {
            addonResponse = response?.body;
        }).catch(function (error) {
            // Not ideal, but API deletes the record returning a 404 when deprovisioned.
            if (error.statusCode === 404 || error.http?.statusCode === 404) {
                addonResponse.state = 'deprovisioned';
            }
            else {
                throw error;
            }
        });
    }
    core_1.ux.action.stop();
    return addonResponse;
};
exports.waitForAddonDeprovisioning = waitForAddonDeprovisioning;
