"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tunnel = exports.trapAndForwardSignalsToChildProcess = void 0;
exports.exec = exec;
exports.execFile = execFile;
exports.interactive = interactive;
exports.runThroughOneOffDyno = runThroughOneOffDyno;
exports.psqlQueryOptions = psqlQueryOptions;
exports.runWithTunnel = runWithTunnel;
exports.psqlFileOptions = psqlFileOptions;
exports.psqlInteractiveOptions = psqlInteractiveOptions;
exports.execPSQL = execPSQL;
exports.consumeStream = consumeStream;
exports.waitForPSQLExit = waitForPSQLExit;
exports.fetchVersion = fetchVersion;
const core_1 = require("@oclif/core");
const debug_1 = __importDefault(require("debug"));
const fs = __importStar(require("fs"));
const node_child_process_1 = require("node:child_process");
const node_events_1 = require("node:events");
const path = __importStar(require("node:path"));
const node_stream_1 = require("node:stream");
const promises_1 = require("node:stream/promises");
const bastion_1 = require("./bastion");
const dyno_1 = __importDefault(require("./dyno"));
const pgDebug = (0, debug_1.default)('pg');
async function exec(db, query, cmdArgs = []) {
    const configs = (0, bastion_1.getConfigs)(db);
    const options = psqlQueryOptions(query, configs.dbEnv, cmdArgs);
    return runWithTunnel(db, configs.dbTunnelConfig, options);
}
async function execFile(db, file) {
    const configs = (0, bastion_1.getConfigs)(db);
    const options = psqlFileOptions(file, configs.dbEnv);
    return runWithTunnel(db, configs.dbTunnelConfig, options);
}
async function interactive(db) {
    const name = db.attachment.name;
    const prompt = `${db.attachment.app.name}::${name}%R%# `;
    const configs = (0, bastion_1.getConfigs)(db);
    configs.dbEnv.PGAPPNAME = 'psql interactive'; // default was 'psql non-interactive`
    const options = psqlInteractiveOptions(prompt, configs.dbEnv);
    return runWithTunnel(db, configs.dbTunnelConfig, options);
}
async function runThroughOneOffDyno(opts) {
    const dyno = new dyno_1.default(opts);
    try {
        await dyno.start();
    }
    catch (error) {
        if (error.exitCode) {
            core_1.ux.error(error.message, { code: error.exitCode, exit: error.exitCode });
        }
        else {
            throw error;
        }
    }
}
function psqlQueryOptions(query, dbEnv, cmdArgs = []) {
    pgDebug('Running query: %s', query.trim());
    const psqlArgs = ['-c', query, '--set', 'sslmode=require', ...cmdArgs];
    const childProcessOptions = {
        stdio: ['ignore', 'pipe', 'inherit'],
    };
    return {
        dbEnv,
        psqlArgs,
        childProcessOptions,
    };
}
async function runWithTunnel(db, tunnelConfig, options) {
    const tunnel = await Tunnel.connect(db, tunnelConfig);
    pgDebug('after create tunnel');
    const psql = execPSQL(options);
    // interactive opens with stdio: 'inherit'
    // which gives the child process the same stdin,stdout,stderr of the node process (global `process`)
    // https://nodejs.org/api/child_process.html#child_process_options_stdio
    // psql.stdout will be null in this case
    // return a string for consistency but ideally we should return the child process from this function
    // and let the caller decide what to do with stdin/stdout/stderr
    const stdoutPromise = psql.stdout ? consumeStream(psql.stdout) : Promise.resolve('');
    const cleanupSignalTraps = (0, exports.trapAndForwardSignalsToChildProcess)(psql);
    try {
        pgDebug('waiting for psql or tunnel to exit');
        // wait for either psql or tunnel to exit;
        // the important bit is that we ensure both processes are
        // always cleaned up in the `finally` block below
        await Promise.race([
            waitForPSQLExit(psql),
            tunnel.waitForClose(),
        ]);
    }
    catch (error) {
        pgDebug('wait for psql or tunnel error', error);
        throw error;
    }
    finally {
        pgDebug('begin tunnel cleanup');
        cleanupSignalTraps();
        tunnel.close();
        kill(psql, 'SIGKILL');
        pgDebug('end tunnel cleanup');
    }
    return stdoutPromise;
}
function psqlFileOptions(file, dbEnv) {
    pgDebug('Running sql file: %s', file.trim());
    const childProcessOptions = {
        stdio: ['ignore', 'pipe', 'inherit'],
    };
    const psqlArgs = ['-f', file, '--set', 'sslmode=require'];
    return {
        dbEnv,
        psqlArgs,
        childProcessOptions,
    };
}
function psqlInteractiveOptions(prompt, dbEnv) {
    let psqlArgs = ['--set', `PROMPT1=${prompt}`, '--set', `PROMPT2=${prompt}`];
    const psqlHistoryPath = process.env.HEROKU_PSQL_HISTORY;
    if (psqlHistoryPath) {
        if (fs.existsSync(psqlHistoryPath) && fs.statSync(psqlHistoryPath).isDirectory()) {
            const appLogFile = `${psqlHistoryPath}/${prompt.split(':')[0]}`;
            pgDebug('Logging psql history to %s', appLogFile);
            psqlArgs = [...psqlArgs, '--set', `HISTFILE=${appLogFile}`];
        }
        else if (fs.existsSync(path.dirname(psqlHistoryPath))) {
            pgDebug('Logging psql history to %s', psqlHistoryPath);
            psqlArgs = [...psqlArgs, '--set', `HISTFILE=${psqlHistoryPath}`];
        }
        else {
            core_1.ux.warn(`HEROKU_PSQL_HISTORY is set but is not a valid path (${psqlHistoryPath})`);
        }
    }
    psqlArgs = [...psqlArgs, '--set', 'sslmode=require'];
    const childProcessOptions = {
        stdio: 'inherit',
    };
    return {
        dbEnv,
        psqlArgs,
        childProcessOptions,
    };
}
function execPSQL({ dbEnv, psqlArgs, childProcessOptions }) {
    const options = {
        env: dbEnv,
        ...childProcessOptions,
    };
    pgDebug('opening psql process');
    const psql = (0, node_child_process_1.spawn)('psql', psqlArgs, options);
    psql.once('spawn', () => pgDebug('psql process spawned'));
    return psql;
}
function consumeStream(inputStream) {
    let result = '';
    const throughStream = new node_stream_1.Stream.PassThrough();
    // eslint-disable-next-line no-async-promise-executor
    const promise = new Promise(async (resolve, reject) => {
        try {
            await (0, promises_1.finished)(throughStream);
            resolve(result);
        }
        catch (error) {
            reject(error);
        }
    });
    // eslint-disable-next-line no-return-assign
    throughStream.on('data', chunk => result += chunk.toString());
    inputStream.pipe(throughStream);
    return promise;
}
// trap SIGINT so that ctrl+c can be used by psql without killing the
// parent node process.
// you can use ctrl+c in psql to kill running queries
// while keeping the psql process open.
// This code is to stop the parent node process (heroku CLI)
// from exiting. If the parent Heroku CLI node process exits, then psql will exit as it
// is a child process of the Heroku CLI node process.
const trapAndForwardSignalsToChildProcess = (childProcess) => {
    const signalsToTrap = ['SIGINT'];
    const signalTraps = signalsToTrap.map(signal => {
        process.removeAllListeners(signal);
        const listener = () => kill(childProcess, signal);
        process.on(signal, listener);
        return [signal, listener];
    });
    // restores the built-in node ctrl+c and other handlers
    return () => {
        signalTraps.forEach(([signal, listener]) => {
            process.removeListener(signal, listener);
        });
    };
};
exports.trapAndForwardSignalsToChildProcess = trapAndForwardSignalsToChildProcess;
async function waitForPSQLExit(psql) {
    let errorToThrow = null;
    try {
        const [exitCode] = await (0, node_events_1.once)(psql, 'close');
        pgDebug(`psql exited with code ${exitCode}`);
        if (exitCode > 0) {
            errorToThrow = new Error(`psql exited with code ${exitCode}`);
        }
    }
    catch (error) {
        pgDebug('psql process error', error);
        const { code } = error;
        if (code === 'ENOENT') {
            errorToThrow = new Error('The local psql command could not be located. For help installing psql, see https://devcenter.heroku.com/articles/heroku-postgresql#local-setup');
        }
    }
    if (errorToThrow) {
        throw errorToThrow;
    }
}
// According to node.js docs, sending a kill to a process won't cause an error
// but could have unintended consequences if the PID gets reassigned:
// https://nodejs.org/docs/latest-v14.x/api/child_process.html#child_process_subprocess_kill_signal
// To be on the safe side, check if the process was already killed before sending the signal
function kill(childProcess, signal) {
    if (!childProcess.killed) {
        pgDebug('killing psql child process');
        childProcess.kill(signal);
    }
}
// a small wrapper around tunnel-ssh
// so that other code doesn't have to worry about
// whether there is or is not a tunnel
class Tunnel {
    bastionTunnel;
    events;
    constructor(bastionTunnel) {
        this.bastionTunnel = bastionTunnel;
        this.events = new node_events_1.EventEmitter();
    }
    async waitForClose() {
        if (this.bastionTunnel) {
            try {
                pgDebug('wait for tunnel close');
                await (0, node_events_1.once)(this.bastionTunnel, 'close');
                pgDebug('tunnel closed');
            }
            catch (error) {
                pgDebug('tunnel close error', error);
                throw new Error('Secure tunnel to your database failed');
            }
        }
        else {
            pgDebug('no bastion required; waiting for fake close event');
            await (0, node_events_1.once)(this.events, 'close');
        }
    }
    close() {
        if (this.bastionTunnel) {
            pgDebug('close tunnel');
            this.bastionTunnel.close();
        }
        else {
            pgDebug('no tunnel necessary; sending fake close event');
            this.events.emit('close', 0);
        }
    }
    static async connect(db, tunnelConfig) {
        const tunnel = await (0, bastion_1.sshTunnel)(db, tunnelConfig);
        return new Tunnel(tunnel);
    }
}
exports.Tunnel = Tunnel;
async function fetchVersion(db) {
    const output = await exec(db, 'SHOW server_version', ['-X', '-q']);
    return output.match(/\d+\.\d+/)?.[0];
}
