"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmbiguousError = exports.NotFound = exports.appAttachment = void 0;
const attachmentHeaders = {
    Accept: 'application/vnd.heroku+json; version=3.actions',
    'Accept-Inclusion': 'addon:plan,config_vars',
};
const appAttachment = async (heroku, app, id, options = {}) => {
    const result = await heroku.post('/actions/addon-attachments/resolve', {
        headers: attachmentHeaders, body: { app, addon_attachment: id, addon_service: options.addon_service },
    });
    return singularize('addon_attachment', options.namespace)(result.body);
};
exports.appAttachment = appAttachment;
class NotFound extends Error {
    statusCode = 404;
    id = 'not_found';
    message = 'Couldn\'t find that addon.';
}
exports.NotFound = NotFound;
class AmbiguousError extends Error {
    matches;
    type;
    statusCode = 422;
    message;
    constructor(matches, type) {
        super();
        this.matches = matches;
        this.type = type;
        this.message = `Ambiguous identifier; multiple matching add-ons found: ${matches.map(match => match.name).join(', ')}.`;
    }
    get body() {
        return { id: 'multiple_matches', message: this.message };
    }
}
exports.AmbiguousError = AmbiguousError;
function singularize(type, namespace) {
    return (matches) => {
        if (namespace) {
            matches = matches.filter(m => m.namespace === namespace);
        }
        else if (matches.length > 1) {
            // In cases that aren't specific enough, filter by namespace
            matches = matches.filter(m => !Reflect.has(m, 'namespace') || m.namespace === null);
        }
        switch (matches.length) {
            case 0:
                throw new NotFound();
            case 1:
                return matches[0];
            default:
                throw new AmbiguousError(matches, type ?? '');
        }
    };
}
