"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parsePostgresConnectionString = exports.getConnectionDetails = exports.bastionKeyPlan = void 0;
exports.getConfigVarName = getConfigVarName;
exports.getConfigVarNameFromAttachment = getConfigVarNameFromAttachment;
const color_1 = __importDefault(require("@heroku-cli/color"));
const core_1 = require("@oclif/core");
const debug_1 = __importDefault(require("debug"));
const node_process_1 = require("node:process");
const bastion_1 = require("./bastion");
const pgDebug = (0, debug_1.default)('pg');
const bastionKeyPlan = (addon) => Boolean(addon?.plan?.name?.split(':')[1].startsWith('private') && addon?.plan?.name?.startsWith('heroku-postgresql'));
exports.bastionKeyPlan = bastionKeyPlan;
function getConfigVarName(configVars, pool = '') {
    const connStringVars = configVars.filter(cv => (cv.endsWith(`${pool ? `_${pool.toUpperCase()}` : ''}_URL`)));
    if (connStringVars.length === 0)
        throw new Error('Database URL not found for this addon');
    return connStringVars[0];
}
const getConnectionDetails = (attachment, configVars = {}, pool = '') => {
    const connStringVar = getConfigVarNameFromAttachment(attachment, configVars, pool);
    // remove _URL from the end of the config var name and add pool if requested
    const baseName = connStringVar.slice(0, -4);
    // build the default payload for non-bastion dbs
    pgDebug(`Using "${connStringVar}" to connect to your database…`);
    const conn = (0, exports.parsePostgresConnectionString)(configVars[connStringVar]);
    const payload = {
        user: conn.user,
        password: conn.password,
        database: conn.database,
        host: conn.host,
        port: conn.port,
        pathname: conn.pathname,
        url: conn.url,
        attachment,
        connStringVar,
    };
    // If bastion creds exist, graft it into the payload
    const bastion = (0, bastion_1.getBastion)(configVars, baseName);
    if (bastion) {
        Object.assign(payload, bastion);
    }
    return payload;
};
exports.getConnectionDetails = getConnectionDetails;
function getConfigVarNameFromAttachment(attachment, config = {}, pool = '') {
    const configVars = attachment.config_vars?.filter((cv) => {
        return config[cv]?.startsWith('postgres://');
    }) ?? [];
    if (configVars.length === 0) {
        core_1.ux.error(`No config vars found for ${attachment.name}; perhaps they were removed as a side effect of ${color_1.default.cmd('heroku rollback')}? Use ${color_1.default.cmd('heroku addons:attach')} to create a new attachment and then ${color_1.default.cmd('heroku addons:detach')} to remove the current attachment.`);
    }
    const configVarName = `${attachment.name}${pool ? `_${pool.toUpperCase()}` : ''}_URL`;
    if (configVars.includes(configVarName) && configVarName in config) {
        return configVarName;
    }
    return getConfigVarName(configVars, pool);
}
const parsePostgresConnectionString = (db) => {
    const dbPath = /:\/\//.test(db) ? db : `postgres:///${db}`;
    const url = new URL(dbPath);
    const { username, password, hostname, pathname, port } = url;
    return {
        user: username,
        password,
        database: pathname.charAt(0) === '/' ? pathname.slice(1) : pathname,
        host: hostname,
        port: port || node_process_1.env.PGPORT || (hostname && '5432'),
        pathname,
        url: dbPath,
    };
};
exports.parsePostgresConnectionString = parsePostgresConnectionString;
